<?php
/*======================================================================*\
|| #################################################################### ||
|| # Invite Add-on  2.1.2
|| # Build: 46
|| # ---------------------------------------------------------------- # ||
|| # Copyright 2012 Daniel Fatkic aka ragtek. All Rights Reserved.   # ||
|| # This file may not be redistributed in whole or significant part. # ||
|| # http://www.ragtek.org                                            # ||
|| #################################################################### ||
\*======================================================================*/


    class Ragtek_Invite_DataWriter_Invite extends XenForo_DataWriter
    {

        protected function _getFields()
        {
            $visitor = XenForo_Visitor::getInstance();
            return array(
                'ragtek_invite' => array(
                    'invite_id' => array('type' => self::TYPE_UINT, 'autoIncrement' => true),
                    'user_id' => array('type' => self::TYPE_UINT, 'default' => $visitor['user_id']),
                    'date' => array('type' => self::TYPE_UINT, 'default' => XenForo_Application::$time),
                    'invited_mail' => array('type' => self::TYPE_STRING, 'maxLength' => 120,
                                            'verification' => array('$this', '_verifyEmail'),
                                            'requiredError' => 'please_enter_valid_email'),
                    'registered_user_id' => array('type' => self::TYPE_UINT, 'default' => 0),
                    'registration_code' => array('type' => self::TYPE_STRING, 'default' => XenForo_Application::generateRandomString(16))
                )
            );
        }

        protected function _postSave()
        {
            if ($this->isInsert()) {
                $this->_decreaseInvites();
            }
        }

        protected function _decreaseInvites()
        {
            if (XenForo_Application::get('options')->ragtek_invite_decreaseInvites)
            {
                $userDw = $this->_getUserDM();
                $userDw->setExistingData($this->get('user_id'));
                $userDw->set('ragtek_free_invites', $userDw->get('ragtek_free_invites') - 1);
                $userDw->save();
            }
        }

        protected function _verifyEmail($email)
        {
            if (!Zend_Validate::is($email, 'EmailAddress')) {
                $this->error(new XenForo_Phrase('please_enter_valid_email'), 'email');
                return false;
            }

            $existingUser = $this->_getUserModel()->getUserByEmail($email);
            if ($existingUser && $existingUser['user_id'] != $this->get('user_id')) {
                $this->error(new XenForo_Phrase('email_addresses_must_be_unique'), 'email');
                return false;
            }

            if ($this->_getInviteModel()->checkIfAllreadyInvited($email)) {
                $this->error(new XenForo_Phrase('ragtek_invite_allready_invited'), 'email');
                return false;
            }

            if (XenForo_Helper_Email::isEmailBanned($email)) {
                $this->error(new XenForo_Phrase('email_address_you_entered_has_been_banned_by_administrator'), 'email');
                return false;
            }

            return true;
        }


        protected function _getExistingData($data)
        {

            if (!$id = $this->_getExistingPrimaryKey($data)) {
                return false;
            }
            $invite = $this->_getInviteModel()->getInviteFromId($id);

            return array('ragtek_invite' => $invite);

        }

        protected function _getUpdateCondition($tableName)
        {
            return 'invite_id = ' . $this->_db->quote($this->getExisting('invite_id'));
        }

        /**
         *
         * @return Ragtek_Invite_Model_Invite
         */
        protected function _getInviteModel()
        {
            return $this->getModelFromCache('Ragtek_Invite_Model_Invite');
        }


        /**
         * @return XenForo_DataWriter_User
         */
        protected function _getUserDM()
        {
            return XenForo_DataWriter::create('XenForo_DataWriter_User');
        }
    }