<?php
/*======================================================================*\
|| #################################################################### ||
|| # Invite Add-on  2.1.2
|| # Build: 46
|| # ---------------------------------------------------------------- # ||
|| # Copyright 2012 Daniel Fatkic aka ragtek. All Rights Reserved.   # ||
|| # This file may not be redistributed in whole or significant part. # ||
|| # http://www.ragtek.org                                            # ||
|| #################################################################### ||
\*======================================================================*/


class Ragtek_Invite_Install
{

    static $minXFVersion = 1010034;


    public function _installVersion41()
    {
        $queries['content_type'] = "
                INSERT INTO xf_content_type (
                    content_type ,
                   addon_id ,
                   fields
                    )
                VALUES (
                    'invite', 'ragtekInvite', ''
                )
                ";

        $queries['content_type_data'] = "
                       INSERT INTO xf_content_type_field
                       (
                           content_type ,            field_name ,            field_value
                       )
                       VALUES
                      ('invite', 'stats_handler_class', 'Ragtek_Invite_StatsHandler')
                      ";

        $this->runQuery($queries);
        self::_end();
    }


    public function _installVersion40()
    {
        if (XenForo_Application::$versionId < self::$minXFVersion) {
            throw new XenForo_Exception('this add-on version was coded for xf 1.1 beta3 or higher', true);
        }
    }

    public function _installVersion35()
    {
        $query = array();
        if (!self::checkIfExist('xf_user', 'ragtek_referrer_userid')) {
            $query['add_referrer'] = "
                ALTER TABLE `xf_user` ADD `ragtek_referrer_userid` INT NOT NULL DEFAULT '0'
            ";
        }

        $this->runQuery($query);
    }


    public function _installVersion25()
    {
        $query = array();
        if (self::checkIfExist('xf_user', 'ragtek_free_invites')) {
            $query['alterInvites'] = "
                ALTER TABLE `xf_user` CHANGE `ragtek_free_invites` `ragtek_free_invites` INT( 10 ) UNSIGNED NOT NULL DEFAULT '0'
            ";
        }
        else {
            $query['alterInvites'] = "
                ALTER TABLE `xf_user` ADD  `ragtek_free_invites` INT( 10 ) UNSIGNED NOT NULL DEFAULT '0'
            ";
        }
        $this->runQuery($query);

    }


    public function _installVersion1()
    {
        $db = XenForo_Application::getDb();
        $installquery = 'CREATE TABLE IF NOT EXISTS `ragtek_invite` (
                            `invite_id` INT( 10 ) NOT NULL AUTO_INCREMENT ,
                            `user_id` INT( 10 ) NOT NULL ,
                            `date` INT( 10 ) NOT NULL ,
                            `invited_mail` VARCHAR( 120 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
                            `registered_user_id` INT( 10 ) NOT NULL ,
                            PRIMARY KEY (  `invite_id` )
                            ) ENGINE = MYISAM ;';
        $db->query($installquery);
        if (!self::checkIfExist('ragtek_invite', 'registration_code')) {
            $addQuery = "ALTER TABLE  `ragtek_invite` ADD  `registration_code` VARCHAR( 16 ) NOT NULL";
            $db->query($addQuery);
        }
    }


    /**
     * @static
     * @param  $existingAddOn
     * @param  $addOnData
     * @return void
     */
    public static function install($existingAddOn, $addOnData)
    {
        $startVersionId = 1;
        $endVersionId = $addOnData['version_id'];

        if ($existingAddOn) {
            $startVersionId = $existingAddOn['version_id'] + 1;
        }
        $install = self::getInstance();

        for ($i = $startVersionId; $i <= $endVersionId; $i++)
        {
            $method = '_installVersion' . $i;
            if (method_exists($install, $method) === false) {
                continue;
            }
            $install->$method();
        }
    }


    static private $instance = null;

    public static function getInstance()
    {

        if (self::$instance == null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    static public function runQuery(array $queries)
    {
        $db = XenForo_Application::getDb();

        foreach ($queries AS $id => $query) {
            $db->query($query);
        }
    }

    static public function uninstall()
    {
        $queries = array();

        $queries['drop_invite_table'] = "
        DROP TABLE ragtek_invite
        ";
        if (self::checkIfExist('xf_user', 'ragtek_free_invites')) {
            $queries['drop_invites'] = "
                   ALTER TABLE xf_user
                   DROP COLUMN ragtek_free_invites
            ";
        }

        $queries['content_type_data'] = "
                       DELETE FROM xf_content_type_field
                       WHERE   content_type = 'invite'
                   ";


        $queries['delete_content_type'] = "
           		    DELETE FROM xf_content_type
           		    WHERE addon_id = 'ragtekInvite'
           		";


        self::runQuery($queries);
    }

    protected static function _end()
    {
        XenForo_Model::create('XenForo_Model_ContentType')->rebuildContentTypeCache();
    }




    public static function checkIfExist($table, $field)
    {
        $db = XenForo_Application::get('db');
        if ($db->fetchRow('SHOW columns FROM `' . $table . '` WHERE Field = ?', $field)) {
            return true;
        }
        else {
            return false;
        }
    }


}