<?php
/*======================================================================*\
|| #################################################################### ||
|| # Invite Add-on  2.1.2
|| # Build: 46
|| # ---------------------------------------------------------------- # ||
|| # Copyright 2012 Daniel Fatkic aka ragtek. All Rights Reserved.   # ||
|| # This file may not be redistributed in whole or significant part. # ||
|| # http://www.ragtek.org                                            # ||
|| #################################################################### ||
\*======================================================================*/


class Ragtek_Invite_Model_Invite extends XenForo_Model
{

    const COOKIENAME = 'inviteCode';
    const TABLE_INVITE = 'ragtek_invite';
    const REFERRER_COOKIENAME = 'ragtek_referrer';

    public function countAllInvites()
    {
        $query = "
            SELECT COUNT(*)
			FROM " . self::TABLE_INVITE . "
	    ";
        return $this->_getDb()->fetchOne($query);
    }

    public function getAllInvites()
    {
        $query = "
        SELECT invites.*,
                 invites.user_id AS reciver,
               user.username as sender
        FROM " . self::TABLE_INVITE . " as invites
        LEFT JOIN  xf_user as user on (user.user_id = invites.user_id)
        ";

        return $this->fetchAllKeyed($query, 'invite_id');
    }

    public function changeInvitePending($code, $userId)
    {
        $invite = $this->getInviteFromCode($code);

        $dw = $this->_getInviteDW();
        $dw->setExistingData($invite);
        $dw->set('registered_user_id', $userId);
        $dw->save();
        $this->deleteCookie();
    }

    /**
     * fetch the invite infos from an id
     * @param  $id
     * @return array
     */
    public function getInviteFromId($id)
    {
        $query = "
            SELECT *
			FROM " . self::TABLE_INVITE . "
			WHERE invite_id = ?
	    ";
        $existing = $this->_getDb()->fetchRow($query, $id);
        return $existing;
    }


    public function getInviteFromCode($code)
    {
        $query = "
            SELECT *
			FROM " . self::TABLE_INVITE . "
			WHERE registration_code = ?
	    ";
        $existing = $this->_getDb()->fetchRow($query, $code);

        return $existing;
    }

    protected function isInvitePending($invite)
    {
        if ($invite['registered_user_id'] == 0) {
            return true;
        }
        else
        {
            return false;

        }
    }

    public function verifyInviteCode($code = null)
    {
        if ($code == NULL) {
            $code = $this->getCookie();
        }
        if (!$code) {
            return false;
        }


        $existing = $this->getInviteFromCode($code);

        if (!$existing) {
            return false;
        }
        // make sure invite wasn't used already
        if ($this->isInvitePending($existing)) {
            $this->setCookie($code);
            return true;
        }
        else {
            return false;
        }
    }

    public function countInvitedUsers($userId)
    {
        $query = "
            SELECT COUNT(*)
			FROM " . self::TABLE_INVITE . "
			WHERE user_id = ?
	    ";
        return $this->_getDb()->fetchOne($query, $userId);
    }

    public function getFreeInvites($visitor)
    {
        return $visitor['ragtek_free_invites'];
    }

    public function checkIfAllreadyInvited($mail)
    {
        $query = "
            SELECT invite.*
            FROM " . self::TABLE_INVITE . " AS invite
            WHERE invite.invited_mail = ?
        ";

        return $this->_getDb()->fetchRow($query, $mail);
    }

    public function getInvitedUsers($userId, $onlyRegistered = false)
    {
        $query = "
                SELECT invite.*,
                user.username,
                user.register_date AS reg_date
                FROM " . self::TABLE_INVITE . " AS invite
                LEFT JOIN xf_user AS user ON (user.user_id = invite.registered_user_id)
                WHERE invite.user_id = ?
        ";

        if ($onlyRegistered == true) {
            $query .= " AND registered_user_id <> 0";
        }
        $query .= " ORDER BY date DESC";

        $users = $this->_getDb()->fetchAll($query, $userId);
        $return = array();
        $days = XenForo_Application::get('options')->ragtek_invite_validdays;

        foreach ($users AS $user) {

            if ($user['reg_date'] != 0) {
                $user['user_id'] = $user['registered_user_id'];
                $user['reg_date'] = XenForo_Locale::getFormattedDate($user['reg_date'], 'd-m-Y');
                $user['status'] = $user['username'];
                $user['showLink'] = true;
            }
            else {
                $user['reg_date'] = '';
                $user['expiredate'] = $user['date'] + ($days * 64000);
                $user['status'] = 'pending';
                $user['statuslink'] = '';
                $user['showLink'] = false;
            }
            $return[] = $user;

        }

        if (count($return) > 0) {
            return $return;
        }
        else {
            return false;
        }
    }

    public function createAndSendInvite($name, $mail, $message = '')
    {
        $visitor = XenForo_Visitor::getInstance();
        if ($visitor['ragtek_free_invites'] > 0) {
            $db = $this->_getDb();
            XenForo_Db::beginTransaction($db);
            $dw = $this->_getInviteDW();
            $dw->set('invited_mail', $mail);
            $dw->save();
            $inviteCode = $dw->get('registration_code');

            $this->sendInviteMail($name, $mail, $inviteCode, $message);

            XenForo_Db::commit($db);
            return true;
        }
        else {
            return false;
        }
    }

    protected function sendInviteMail($name, $mail, $inviteCode, $message = '')
    {
        $user = XenForo_Visitor::getInstance();

        $mailParams = array(
            'name'              => $name,
            'inviteCode'        => $inviteCode,
            'inviter'           => $user->get('username'),
            'boardTitle'        => XenForo_Application::get('options')->boardTitle,
            'additionalMessage' => $message
        );
        $inviteMail = XenForo_Mail::create('ragtek_inviteMail', $mailParams);
        $inviteMail->send($mail, $name);

    }

    /**
     * @param  $visitor
     * @return check if the user have free invites
     */
    public function canInvite($visitor)
    {
        return $visitor['ragtek_free_invites'] > 0 ? true : false;
    }


    /**
     * @return Ragtek_Invite_DataWriter_Invite
     */
    protected function _getInviteDW()
    {
        return XenForo_DataWriter::create('Ragtek_Invite_DataWriter_Invite');
    }


    /**
     * @static
     * @return array|false|string
     */
    public static function getCookie()
    {
        return XenForo_Helper_Cookie::getCookie(self::COOKIENAME);
    }


    /**
     * @static
     * @param  $code
     * @return void
     */
    public static function setCookie($code)
    {
        XenForo_Helper_Cookie::setCookie(self::COOKIENAME, $code);
    }

    /**
     * delete the invite cookie after the successfull registration
     */
    public function deleteCookie()
    {
        XenForo_Helper_Cookie::deleteCookie(self::COOKIENAME);
    }

    public function canSeeTab()
    {
        return Ragtek_Invite_Helper::perm('ragtek_InviteCanSeeTab');
    }

    /**
     * @deprecated this is now handeled in the dw
     * @param  $userId
     * @return void
     */
    protected function _removeInvite($userId)
    {
        $dw = $this->_getUserDW();
        $dw->setExistingData($userId);
        $dw->set('ragtek_free_invites', $dw->get('ragtek_free_invites') - 1);
        $dw->save();
    }

    /**
     * @return XenForo_DataWriter_User
     */
    protected function _getUserDW()
    {
        return XenForo_DataWriter::create('XenForo_DataWriter_User');
    }

    public function canDeleteInvite()
    {
        return Ragtek_Invite_Helper::perm('canDelete');
    }

    public function getInviteForUserId($userId)
    {
        $query = "
        SELECT * from " . self::TABLE_INVITE . "
        WHERE registered_user_id = ?
        ";
        return $this->_getDb()->fetchRow($query, $userId);
    }


    public function deleteInviteForUserId($userId)
    {
        $invite = $this->getInviteForUserId($userId);
        if ($invite) {
            $dw = $this->_getInviteDW();
            $dw->setExistingData($invite);
            $dw->delete();
        }
    }

    public function deleteInvite($inviteId)
    {

        if ($this->canDeleteInvite()
                AND $invite = $this->getInviteFromId($inviteId)
        ) {
            $dw = $this->_getInviteDW();
            $dw->setExistingData($invite);
            $dw->delete();
        }
    }

    public function addInvites($groupId, $invites, $onlyPrimary)
    {
        $users = $this->_getUserIds($groupId, $onlyPrimary);

        $this->_addInvites($users, $invites);

    }

    protected function _addInvites(array $users, $invites)
    {
        if (count($users) > 0) {

            foreach ($users AS $user) {
                $userArray[] = $user['user_id'];
            }
            $db = $this->_getDb();
            $userIdsQuoted = $db->quote($userArray);

            $db->update('xf_user',
                array('ragtek_free_invites' => $invites),
                    'user_id IN (' . $userIdsQuoted . ')');
        }

    }

    /**
     * @return XenForo_Model_User
     */
    protected function _getUserModel()
    {
        return $this->getModelFromCache('XenForo_Model_User');
    }

    /**
     * @param $groupId
     * @param bool $both
     * @return array
     */
    protected function _getUserIds($groupId, $both = false)
    {
        $db = $this->_getDb();
        if (!$both) {
            $query = "select user_id
                    FROM xf_user
                    WHERE
                    user_group_id = ?
                    OR
                        secondary_group_ids IN (" . $db->quote($groupId) . ")
                    ";
        }
        else
        {
            $query = "
                    SELECT user_id
                    FROM xf_user
                    WHERE
                    user_group_id = ?
            ";
        }
        return $this->fetchAllKeyed($query, 'user_id', $groupId);
    }


    public function setUserInvites($userId, $newInvitesValue)
    {
        $dataWriter = $this->_getUserDW();
        $dataWriter->setExistingData($userId);
        $dataWriter->set('ragtek_free_invites', $newInvitesValue);
        $dataWriter->save();
    }


    public function getReferredUsers($userId)
    {
        $query = "
        SELECT * FROM xf_user
        WHERE ragtek_referrer_userid = ?
        ";

        return $this->_getDb()->fetchAll($query, $userId);
    }

    public function setReferrer($userId, $referrerId)
    {
        $dw = $this->_getUserDW();
        $dw->setExistingData($userId);
        $dw->set('ragtek_referrer_userid', $referrerId);
        $dw->save();
        $this->_deleteReferrerCookie();
    }

    protected function _deleteReferrerCookie()
    {
        XenForo_Helper_Cookie::deleteCookie(self::REFERRER_COOKIENAME);
    }

    public function setReferrerCookie($userId)
    {
        $user = $this->_getUserModel()->getUserById($userId);
        XenForo_Helper_Cookie::setCookie('ragtek_referrer', $user['user_id']);
    }

    public function countReferredUsers($userId)
    {
        $query = "
                        SELECT COUNT(*)
            			FROM xf_user
            			WHERE ragtek_referrer_userid = ?
            	    ";
        return $this->_getDb()->fetchOne($query, $userId);
    }
}