<?php
/*======================================================================*\
|| #################################################################### ||
|| # Invite Add-on  2.1.2
|| # Build: 46
|| # ---------------------------------------------------------------- # ||
|| # Copyright 2012 Daniel Fatkic aka ragtek. All Rights Reserved.   # ||
|| # This file may not be redistributed in whole or significant part. # ||
|| # http://www.ragtek.org                                            # ||
|| #################################################################### ||
\*======================================================================*/


class Ragtek_Invite_StaticMethods
{

    static $haveInvitedUsers = false;
    static $invitedUsers = null;
    static $showMemberTab = false;


    public static function setReferrerCookie(XenForo_Controller $controller, $action)
    {
        if (!XenForo_Visitor::getUserId()
                AND
                !XenForo_Helper_Cookie::getCookie(Ragtek_Invite_Model_Invite::REFERRER_COOKIENAME) AND
                        $referrer = $controller->getRequest()->get('ref')
        ) {
            XenForo_Model::create('Ragtek_Invite_Model_Invite')->setReferrerCookie($referrer);
        }
    }


    public static function createTemplate(&$templateName, &$params, $template)
    {

        switch ($templateName) {
            case 'ragtek_invite':
            case 'account_wrapper':
                $template->preloadTemplate('ragtek_invite_sidebarblock');
                break;
            case 'member_view':
                $template->preloadTemplate('ragtek_invite_memberTab');
                $template->preloadTemplate('ragtek_invite_membercontent');
                break;
        }
        $template->preloadTemplate('ragtek_invite_visitortab');
    }


    public static function templateHook($name, &$contents, array $params, XenForo_Template_Abstract $template)
    {
        switch ($name) {
            case 'account_wrapper_sidebar':

                $templateParams = $template->getParams();
                $search = '<!-- slot: pre_conversations -->';
                $params = array(
                    'selectedKey' => $templateParams['selectedKey']
                );

                $inviteBlock = $template->create('ragtek_invite_sidebarblock', $params)->render();

                $contents = str_replace($search, $search . $inviteBlock, $contents);
                break;

            case 'member_view_tabs_heading':

                $inviteModel = XenForo_Model::create('Ragtek_Invite_Model_Invite');
                if (
                    (
                            (self::$invitedUsers = $inviteModel->getInvitedUsers($params['user']['user_id'], true))
                            > 0
                    )
                    AND
                    (Ragtek_Invite_Helper::perm('ragtek_InviteCanSeeTab'))
                ) {
                    self::$haveInvitedUsers = true;
                    $inviteTab = $template->create('ragtek_invite_memberTab', $template->getParams());
                    $contents .= $inviteTab;
                }
                break;

            case 'admin_user_edit_tabs':
                if (Ragtek_Invite_Helper::canAdmin()) {
                    $contents .= $template->create('ragtek_invite_tab');

                }
                break;
            case 'admin_user_edit_panes':
                if (Ragtek_Invite_Helper::canAdmin()) {
                    $contents .= $template->create('ragtek_invite_userpage', $template->getParams());

                }
                break;
            case 'user_criteria_content':
                # $contents .=$template->create('ragtek_invite_criteria_user');

        }

        // add the content for the invited users tab
        if ($name == 'member_view_tabs_content' AND self::$haveInvitedUsers) {

            $invitedUsers = $template->create('ragtek_invite_membercontent'
                , array(
                    'invitedUsers'        => self::$invitedUsers,
                    'invitedUsersCounter' => count(self::$invitedUsers)
                ));
            $contents .= $invitedUsers;
        }

        if ($name == 'navigation_visitor_tab_links1') {
            if (Ragtek_Invite_Helper::perm('canUse')) {

                $contents .= $template->create('ragtek_invite_visitortab');
            }
        }
    }


    /**
     * @static
     * @param array $extraTabs
     * @param  $selectedTabId
     * @return void
     */
    public static function addNavbarTab(array &$extraTabs, $selectedTabId)
    {
        if (Ragtek_Invite_Helper::perm('canUse')
                AND
                XenForo_Application::get('options')->ragtek_invite_showNavLink
        ) {
            $extraTabs['invites'] = array(
                'title' => new XenForo_Phrase('ragtek_invite_navbartab'),
                'href'  => XenForo_Link::buildPublicLink('invites')
            );
        }
    }


    /**
     * @static
     * @param  $class
     * @param  $extend
     * @return void
     */
    public static function extendUserDW($class, &$extend)
    {
        if ($class == 'XenForo_DataWriter_User') {
            $extend[] = 'Ragtek_Invite_DataWriter_User';
        }
    }

    /**
     * @static
     * @param  $class
     * @param  $extend
     * @return void
     */
    public static function listener($class, &$extend)
    {
        if ($class == 'XenForo_ControllerPublic_Register') {
            $extend[] = 'Ragtek_Invite_ControllerPublic_Register';
        }
        if ($class == 'XenForo_ControllerPublic_Member') {
            $extend[] = 'Ragtek_Invite_ControllerPublic_Member';
        }

        if ($class == 'XenForo_ControllerAdmin_User') {
            $extend[] = 'Ragtek_Invite_ControllerAdmin_User';
        }
    }

    public static function fileHashes(XenForo_ControllerAdmin_Abstract $controller, array &$hashes)
    {
        $fileHashes = Ragtek_Invite_Hashes::getHashes();
        $hashes += $fileHashes;
    }


    public static function criteriaUser($rule, array $data, array $user, &$returnValue)
    {
        if ($rule == 'ragtek_free_invites') {
            if (isset($user['ragtek_free_invites'])
                    AND $user['ragtek_free_invites'] >= $data['invites']
            ) {
                $returnValue = true;
            }
        }
    }
}